{
  drpfiles.pas
    v1.1a of 09.04.99
      Windowed component which provides an area which can accept files dragged
      and dropped from File Manager. Any child controls of the component will
      also be able to accept dropped files
}

{
  History
  - v1.0 of 15.05.98
      Original version
  - v1.1 of 22.10.98
      Changed 2nd parameter to DragQueryFile when used to find number of files
      dropped from $FFFF to Cardinal(-1) to make it portable across 16 and 32
      bit platforms. Thjis was needed to allow the component to compile under
      Delphi 2
  - v1.1a of 09.04.99
      Changed installation palette from "Own" to "PJ Stuff"
}

{
  Dependencies
    None
}

{
  Legal notice
    This component is copyright  1998-9 by Peter Johnson, Erw'r Llan, Llanarth,
    SA47 0RG, Wales, UK. E-Mail peter@erwrllan.freeserve.co.uk
    The component is free-ware and may be distributed freely providing no charge
    is made for it. The source code may be modified and redistributed providing
    that this notice remains intact.
    If the code gets modified and improved on I'd like to see it.
}


unit DrpFiles;

interface

uses
  {Delphi library}
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TDropFilesEvent = procedure(Sender : TObject; X, Y : integer) of object;
    {Event generated when files are dropped on the control - identify sending
    control and the X and Y co-oridnates where files were dropped}

type
  TPJDropFiles = class(TCustomControl)
  private
    {Property storage}
    FDropPoint : TPoint;
    FOnDropFiles : TDropFilesEvent;
    {Other storage}
    FList : TStringList;
      {The list of files dropped}
    {Property access methods}
    function GetCount : integer;
    function GetFile(I : integer) : string;
    function GetDropControl : TControl;
    procedure SetAccept(AFlag : Boolean);
  protected
    procedure CreateWnd; override;
      {Window creation method - registers that window can accept dropped files}
    procedure WMDropFiles(var Msg : TMessage); message WM_DROPFILES;
      {File drop message handler - used to record info about dropped files}
    procedure CMEnabledChanged(var Msg : TMessage); message CM_ENABLEDCHANGED;
      {Message triggered when Enabled property changes - used to toggle whether
      files can be accepted or not}
    procedure DropFiles; dynamic;
      {Triggers OnDropFiles event}
  public
    constructor Create(AOwner : TComponent); override;
      {Control constructor - sets default values of properties and creates owned
      list object}
    destructor Destroy; override;
      {Control destructor - frees owned list object}
    procedure Paint; override;
      {Control paint handler - draws dashed outline to control only when
      designing}
    property Count : integer read GetCount;
      {The number of files dropped}
    property Files[I : integer] : string read GetFile;
      {The names of the dropped files}
    property DropPoint : TPoint read FDropPoint;
      {The mouse coordinates where the last files were dropped}
    property DropControl : TControl read GetDropControl;
      {The child control (if any) under the mouse when files are dropped -
      returns nil if no control under mouse}
  published
    {Inherited protected properties published}
    property Enabled stored True;
      {Enables/disables ability to receive dropped files}
    property Align;
    property ParentShowHint;
    property ShowHint;
    {Inherited published propertes given new defaults}
    property Width default 80;
    property Height default 60;
    {New properties}
    property OnDropFiles : TDropFilesEvent read FOnDropFiles write FOnDropFiles;
      {Event triggered when files have been dropped}
  end;

procedure Register;

implementation

uses
  {Delphi library unit}
  ShellAPI;     {for file drop API calls}

{ --- Public methods --- }

constructor TPJDropFiles.Create(AOwner : TComponent);
  {Control constructor - sets default values of properties and creates owned
  list object}
begin
  inherited Create(AOwner);
  {Make control accept child controls}
  ControlStyle := ControlStyle + [csAcceptsControls];
  {Set default size}
  Width := 80;
  Height := 60;
  {Ensure we're not a tab-stop}
  TabStop := False;
  {Create owned list of files}
  FList := TStringList.Create;
end;

destructor TPJDropFiles.Destroy;
  {Control destructor - frees owned list object}
begin
  {Free owned file list}
  FList.Free;
  inherited Destroy;
end;

procedure TPJDropFiles.Paint;
  {Control paint handler - draws dashed outline to control only when designing}
begin
  {Paint a bounding frame only if designing}
  if csDesigning in ComponentState then
    with Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end
end;

{ --- Protected methods --- }

procedure TPJDropFiles.CreateWnd;
  {Window creation method - registers that window can accept dropped files}
begin
  inherited CreateWnd;
  DragAcceptFiles(Handle, Enabled);
end;

procedure TPJDropFiles.WMDropFiles(var Msg: TMessage);
  {File drop message handler - used to record info about dropped files}
var
  DropH : THandle;                    {Saves handle used for dragging and
                                      dropping}
  NameLen : Word;                     {Length of buffer required for file name}
  NumDropped : Word;                  {Number of files dropped}
  PFileName : array[0..255] of char;  {PString to hold file name}
  I : integer;                        {Loop control for scanning file names}
begin
  {Save handle used for dragging and dropping - to be released later - its in
  the WParam field of the message}
  DropH := Msg.WParam;
  try
    {Clear string list}
    FList.Clear;
    {Find number of files dropped}
    NumDropped := DragQueryFile(DropH, Cardinal(-1), nil, 0);
    for I := 0 to NumDropped - 1 do
    begin
      {Find size required for filename buffer}
      NameLen := DragQueryFile(DropH, I, nil, 0);
      {Provide buffer for, and get file name}
      DragQueryFile(DropH, I, PFileName, NameLen + 1);
      {Add file to string list}
      FList.Add(StrPas(PFileName));
    end;
    {Find co-ordinates where files dropped}
    DragQueryPoint(DropH, FDropPoint);
  finally
    {Release handle assoc. with drag/drop}
    DragFinish(DropH);
  end;
  {Trigger event}
  DropFiles;
end;

procedure TPJDropFiles.CMEnabledChanged(var Msg : TMessage);
  {Message triggered when Enabled property changes - used to toggle whether
  files can be accepted or not}
begin
  inherited;
  DragAcceptFiles(Handle, Enabled);
end;

procedure TPJDropFiles.DropFiles;
  {Triggers OnDropFiles event}
begin
  if Assigned(FOnDropFiles) then FOnDropFiles(Self, FDropPoint.X, FDropPoint.Y);
end;

{ --- Private property access methods --- }

function TPJDropFiles.GetCount : integer;
  {Read access method for Count property - gets number of files dropped from
  FList object}
begin
  Result := FList.Count;
end;

function TPJDropFiles.GetFile(I : integer) : string;
  {Read access method for Files array property - returns name of file at the
  given array index}
begin
  Result := FList[I];
end;

function TPJDropFiles.GetDropControl : TControl;
  {Read access method for DropControl property - returns child control which was
  under mouse when files were dropped, or nil if no child control was under
  mouse}
var
  I : integer;      {loop control for scanning thru all child controls}
begin
  {Set default nil (not found) result}
  Result := nil;
  {Iterate across all child controls looking for one under mouse}
  for I := 0 to ControlCount - 1 do
  begin
    {Check if current control was under mouse}
    if PtInRect(Controls[I].BoundsRect, Point(FDropPoint.X, FDropPoint.Y)) then
    begin
      {Found one - record it and quit}
      Result := Controls[I];
      Break;
    end;
  end;
end;

procedure TPJDropFiles.SetAccept(AFlag : Boolean);
begin
end;

{ --- Component registration routine --- }

procedure Register;
begin
  RegisterComponents('PJ Stuff', [TPJDropFiles]);
end;

end.
